@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tclWinReg.c,v 1.21 2003/03/03 17:12:49 dgp Exp $
+ * RCS: @(#) $Id: tclWinReg.c,v 1.21.2.4 2004/09/01 17:26:04 hobbs Exp $
  */
 
 #include <tclPort.h>
@@ -228,7 +228,7 @@ Registry_Init(
     }
 
     Tcl_CreateObjCommand(interp, "registry", RegistryObjCmd, NULL, NULL);
-    return Tcl_PkgProvide(interp, "registry", "1.1.1");
+    return Tcl_PkgProvide(interp, "registry", "1.1.3");
 }
 
 /*
@@ -982,7 +982,7 @@ OpenSubKey(
     keyName = (char *) Tcl_WinUtfToTChar(keyName, -1, &buf);
     if (flags & REG_CREATE) {
 	DWORD create;
-	result = (*regWinProcs->regCreateKeyExProc)(rootKey, keyName, 0, "",
+	result = (*regWinProcs->regCreateKeyExProc)(rootKey, keyName, 0, NULL,
 		REG_OPTION_NON_VOLATILE, mode, NULL, keyPtr, &create);
     } else {
 	if (rootKey == HKEY_PERFORMANCE_DATA) {
@@ -1328,7 +1328,7 @@ BroadcastValue(
     int objc,			/* Number of arguments. */
     Tcl_Obj * CONST objv[])	/* Argument values. */
 {
-    DWORD result, sendResult;
+    LRESULT result, sendResult;
     UINT timeout = 3000;
     int len;
     char *str;
@@ -1362,8 +1362,8 @@ BroadcastValue(
 	    (WPARAM) 0, (LPARAM) str, SMTO_ABORTIFHUNG, timeout, &sendResult);
 
     objPtr = Tcl_NewObj();
-    Tcl_ListObjAppendElement(NULL, objPtr, Tcl_NewIntObj((int) result));
-    Tcl_ListObjAppendElement(NULL, objPtr, Tcl_NewIntObj((int) sendResult));
+    Tcl_ListObjAppendElement(NULL, objPtr, Tcl_NewLongObj((long) result));
+    Tcl_ListObjAppendElement(NULL, objPtr, Tcl_NewLongObj((long) sendResult));
     Tcl_SetObjResult(interp, objPtr);
 
     return TCL_OK;
